@@ -7,18 +7,18 @@ from typing import Callable, Iterable, Type, Union
 
 from ..data_types import (BaseDeltaType, Void,
                           delta_type, make_forked_return,
-                          DComplex, DArray, DRecord, DTuple)
+                          Complex, Array, Record, Tuple)
 from ..runtime import DeltaRuntimeExit
 from ..wiring import (DeltaBlock,
                       DeltaMethodBlock,
                       Interactive,
                       PythonNode,
-                      InteractiveProcess)
+                      InteractiveBodyTemplate)
 
 
 def make_generator(val: Union[object, Iterable],
                    reps: int = None,
-                   verbose: bool = False) -> InteractiveProcess:
+                   verbose: bool = False) -> InteractiveBodyTemplate:
     """Used to create a generator node that
     produces a series of messages of the same data type.
 
@@ -54,7 +54,7 @@ def make_generator(val: Union[object, Iterable],
         >>> generator = dl.lib.make_generator(10, reps=5)
 
         # Receive 5 integers and send their sum
-        >>> @dl.Interactive({"a": int}, int)
+        >>> @dl.Interactive([("a", int)], int)
         ... def accumulator(node):
         ...     memory = []
         ...     for _ in range(5):
@@ -104,7 +104,7 @@ def make_generator(val: Union[object, Iterable],
         elem_type = delta_type(val)
         vals_to_send = (deepcopy(val) for _ in range(reps))
 
-    @Interactive({}, elem_type)
+    @Interactive([], elem_type)
     def generator(node: PythonNode):
         for v in vals_to_send:
             if verbose:
@@ -164,11 +164,11 @@ def make_splitter(t: Union[Type, BaseDeltaType],
 class DeltaJsonEncoder(json.JSONEncoder):
     def default(self, obj):
         obj_type = delta_type(obj)
-        if isinstance(obj_type, DComplex):
+        if isinstance(obj_type, Complex):
             return {"real": obj.real, "imaginary": obj.imag}
-        elif isinstance(obj_type, (DArray, DTuple)):
+        elif isinstance(obj_type, (Array, Tuple)):
             return [self.default(o) for o in obj]
-        elif isinstance(obj_type, DRecord):
+        elif isinstance(obj_type, Record):
             props = obj.__annotations__.keys()
             d = {p: self.default(getattr(obj, p)) for p in props}
             return d
