@@ -5,6 +5,7 @@ nodes and placeholders.
 from __future__ import annotations
 from collections import OrderedDict
 from copy import deepcopy
+import inspect
 from typing import TYPE_CHECKING, Dict, List, Set, Tuple, Union
 import logging
 import textwrap
@@ -18,12 +19,12 @@ from deltalanguage.logging import make_logger
 from ..data_types import (BaseDeltaType,
                           DeltaIOError,
                           DeltaTypeError,
-                          DUnion,
+                          Union,
                           Top)
 from ._node_classes.abstract_node import AbstractNode
 from ._node_classes.node_bodies import Latency, PyConstBody, PyInteractiveBody
 from ._node_classes.real_nodes import OutPort
-from ._node_factories import py_node_factory
+from ._node_templates import NodeTemplate
 
 if TYPE_CHECKING:
     from ._node_classes.placeholder_node import PlaceholderNode
@@ -81,7 +82,7 @@ class DeltaGraph:
 
     Note that this graph has 2 nodes, as a dummy node has been created for
     a constant.
-    Also note that the name of each node has its index appended in the end:    
+    Also note that the name of each node has its index appended in the end:
 
     One can also use the interface with ``matplotlib`` to draw
     the same graph:
@@ -99,8 +100,8 @@ class DeltaGraph:
 
         >>> import deltalanguage as dl
 
-        >>> rt = dl.DeltaPySimulator(graph)
-        >>> rt.run()
+        >>> rt = dl.DeltaPySimulator(graph) # doctest: +SKIP
+        >>> rt.run() # doctest: +SKIP
         42
 
     Node inputs and outputs must be typed (in case of no output
@@ -109,7 +110,7 @@ class DeltaGraph:
 
     .. code-block:: python
 
-        >>> graph.check()
+        >>> graph.check() # doctest: +SKIP
         True
 
     Attributes
@@ -124,7 +125,7 @@ class DeltaGraph:
     global_stack = []
     placeholder_name_index: int = 0
 
-    def __init__(self, name=None, lvl: int = logging.ERROR):
+    def __init__(self, name: str = "main", lvl: int = logging.ERROR):
         self.name = name
         self.placeholders: Dict[str, PlaceholderNode] = {}
         self.nodes: List[RealNode] = []
@@ -152,6 +153,9 @@ class DeltaGraph:
         """End of the graph context, pops from top of graph stack as this
         graph is no longer active.
         """
+        if not self.select_bodies(override=False):
+            self.log.info(f'Graph {self.name} contains a node without a body, '
+                          'please define it before execution.')
         DeltaGraph.global_stack.pop()
 
     def __str__(self) -> str:
@@ -239,6 +243,14 @@ class DeltaGraph:
             def _splitter(to_split: merged_type) -> _SplitterT:
                 return _SplitterC(*(deepcopy(to_split) for _ in range(_reps)))
 
+            # Dynamically annotated function does not get signature set
+            # properly, so to use normal interface we have to manually set it
+            params = [inspect.Parameter('to_split',
+                                        inspect.Parameter.POSITIONAL_OR_KEYWORD,
+                                        annotation=merged_type)]
+            _splitter.__signature__ = inspect.Signature(parameters=params,
+                                                        return_annotation=_SplitterT)
+
             # If merged_node has forked output only get the relevant argument
             # TODO This duplicates PyConstBody.eval and should be refactored
             if out_ports[0].index is None:
@@ -246,18 +258,14 @@ class DeltaGraph:
             else:
                 merged_node_output = getattr(merged_node, out_ports[0].index)
 
-            # create a const splitter node if merged_node is const
-            new_node = py_node_factory(
-                self,
-                merged_node.is_const(),
-                _splitter,
-                OrderedDict([('to_split', merged_type)]),
-                _SplitterT,
-                merged_node_output,
-                name='splitter',
-                latency=Latency(time=100),
-                lvl=merged_node.log.level
-            )
+            # Construct the node using NodeTemplate
+            template = NodeTemplate.merge_deltablock(None,
+                                                     _splitter,
+                                                     merged_node.is_const(),
+                                                     name='splitter',
+                                                     latency=Latency(time=100),
+                                                     lvl=merged_node.log.level)
+            new_node = template.call_with_graph(self, merged_node_output)
 
             # send splitters outputs to the original destinations
             for i, dest in enumerate(all_destinations):
@@ -297,7 +305,7 @@ class DeltaGraph:
             node.out_ports = [self._merge_out_ports(x)
                               for x in out_ports_dict.values()]
 
-    def check(self):
+    def check(self, allow_top: bool = True, allow_node_key: bool = True):
         """Check that the graph is correct.
 
         This is a 2-stage process that is outlined below:
@@ -311,6 +319,15 @@ class DeltaGraph:
 
             2. Check typing of wires, see the impl and characterization tests.
 
+        Parameters
+        ----------
+        allow_top : bool
+            Whether or not input ports are allowed to have type Top().
+        allow_node_key : bool
+            Whether or not nodes are allowed to have a node key
+            (a debugging parameter from which node bodies can send/receive
+            messages).
+
         Returns
         -------
         bool
@@ -360,16 +377,19 @@ class DeltaGraph:
         # nodes with interactive bodies cannot be portless,
         # as they block simulation, until scheduling is implemented
         for node in self.nodes:
-            # after multibody extension: check all bodies
-            if (isinstance(node.body, PyInteractiveBody)
-                    and len(node.in_ports) == 0
-                    and len(node.out_ports) == 0):
+            if not allow_node_key and node.node_key is not None:
                 raise DeltaIOError(
-                    f"Node {node} does not have I/O has an interactive body "
-                    f"{node.body}. At the moment this is not supported.\n"
-                    "Please either remove this body or connect this node "
-                    "to another one."
-                )
+                    "Node has a node key when allow_node_key is set to False")
+            for body in node.bodies:
+                if (isinstance(body, PyInteractiveBody)
+                        and len(node.in_ports) == 0
+                        and len(node.out_ports) == 0):
+                    raise DeltaIOError(
+                        f"Node {node} without any I/O has an interactive body "
+                        f"{body}. At the moment this is not supported.\n"
+                        "Please either remove this body or connect this node "
+                        "to another one."
+                    )
 
         # check missing inputs
         in_ports_unused = [port
@@ -385,22 +405,48 @@ class DeltaGraph:
         for port in out_ports_all:
             try:
                 self.check_wire(type_s=port.port_type,
-                                type_r=port.destination.port_type)
+                                type_r=port.destination.port_type,
+                                allow_top=allow_top)
             except DeltaTypeError as port_type_err:
                 raise DeltaTypeError(
                     port_type_err.args[0] + f'\n{port=}') from port_type_err
 
         return True
 
+    def select_bodies(self,
+                      exclusions: List[object] = [],
+                      preferred: List[object] = [],
+                      override=True):
+        """Shortcut to selecting bodies accross whole graph.
+        Applies exclusion rules then selects the first valid body for each
+        node, preferred bodies selected if available.
+
+        Returns
+        -------
+        bool
+            ``True`` if all nodes now have a ``body``.
+        """
+        all_selected = True
+        for node in self.nodes:
+            is_selected = node.select_body(exclusions, preferred, override)
+            all_selected = all_selected and is_selected
+        return all_selected
+
     @staticmethod
-    def check_wire(type_s: BaseDeltaType, type_r: BaseDeltaType):
+    def check_wire(type_s: BaseDeltaType,
+                   type_r: BaseDeltaType,
+                   allow_top: bool = True):
         """Check that ``type_s`` sent by :py:class:`OutPort` can by received
         as ``type_r`` by :py:class:`InPort`.
 
         In case when ``type_s`` is an instance of
-        :py:class:`DUnion<deltalanguage.data_types.DUnion>`, it is
+        :py:class:`Union<deltalanguage.data_types.Union>`, it is
         enforces that each each type is received, i.e. strict typing.
 
+        If ``allow_top`` is True then the receiver type can be Top(), which
+        accepts any sending type. This is useful when debugging in the Python
+        simulator.
+
         Returns
         -------
         bool
@@ -420,14 +466,17 @@ class DeltaGraph:
                 or (not isinstance(type_r, BaseDeltaType)):
             raise DeltaTypeError(f"Unsupported types\n{type_s=}\n{type_r=}")
 
+        if type_s == Top():
+            raise DeltaTypeError("Sender type cannot be Top().")
+
         if type_r == type_s:
             return True
 
-        if type_r == Top():
-            return True
-
-        if type(type_r) == DUnion and Top() in type_r.elems:
+        if type_r == Top() and allow_top:
             return True
+        elif type_r == Top():
+            raise DeltaTypeError(
+                "Receiving type cannot be Top() when allow_top is False.")
 
         raise DeltaTypeError(f"Broken strict typing\n"
                              f"{type_s=}\n{type_r=}")
@@ -579,15 +628,15 @@ class DeltaGraph:
         for Deltaflow graphs is given in `deltalanguage/DeltaStyle.xml`.
 
         After calling this method, with a path pointing to an empty .gml
-        file, install and open Cytoscape. 
+        file, install and open Cytoscape.
         Select your .gml file from the import dialog via:
         File > Import > Network from file
-        Ensure the 'Network' panel is selected on the left toolbar, in which 
+        Ensure the 'Network' panel is selected on the left toolbar, in which
         you can select which network/graph you are visualising.
         To import a style select your .xml file via:
         File > Import > Style from file
         Along the left toolbar, select the 'Style' panel.
-        In this panel there is a drop-down menu that allows you to pick 
+        In this panel there is a drop-down menu that allows you to pick
         which style you are editing or using to visualise graphs.
         To spread nodes out, you can apply a layout function.
         Layout -> Edge-weighted Spring Embedded Layout -> (none)
