@@ -1,22 +1,26 @@
-import abc
+from abc import abstractmethod
 import atexit
 import logging
 from time import sleep
-from typing import Callable, Type, Union
+from collections import OrderedDict
+from typing import Callable, List, Type, Union
 
 import migen
 
 from deltalanguage.data_types import (BaseDeltaType,
                                       DeltaTypeError,
-                                      DOptional,
+                                      Optional,
+                                      Void,
                                       as_delta_type,
                                       make_forked_return)
-from deltalanguage.wiring._delta_graph import DeltaGraph
-from deltalanguage.wiring._node_factories import py_method_node_factory
 from deltalanguage.logging import make_logger
+from .._body_templates import BodyTemplate
+from .._node_templates import NodeTemplate
+from .latency import Latency
+from .node_bodies import PyMigenBody
 
 
-class MigenNodeTemplate(abc.ABC):
+class MigenNodeTemplate(BodyTemplate):
     """Base class for nodes that use ``migen`` for generation of internal
     logic.
 
@@ -82,8 +86,8 @@ class MigenNodeTemplate(abc.ABC):
         ...
         ...     def migen_body(self, template):
         ...         # define I/O: each port gets `data`, `valid`, and `ready`
-        ...         a = template.add_pa_in_port('a', DOptional(int))
-        ...         b = template.add_pa_in_port('b', DOptional(int))
+        ...         a = template.add_pa_in_port('a', Optional(int))
+        ...         b = template.add_pa_in_port('b', Optional(int))
         ...         out = template.add_pa_out_port('out', int)
         ...
         ...         # LOGIC: identical to `migen.Module` syntax
@@ -133,11 +137,12 @@ class MigenNodeTemplate(abc.ABC):
                  name: str = None,
                  lvl: int = logging.ERROR,
                  vcd_name: str = None,
-                 generics: dict = None):
+                 generics: dict = None,
+                 node_template: NodeTemplate = None,
+                 tags: List[str] = []):
         if name is None:
-            self.name = type(self).__name__
-        else:
-            self.name = name
+            name = type(self).__name__
+        super().__init__(name, Latency(clocks=1), lvl, tags)
 
         self.module_name = self.name
         self.log = make_logger(lvl, f"{self.name}")
@@ -148,8 +153,10 @@ class MigenNodeTemplate(abc.ABC):
 
         # Predefining values to satisfy linting check.
         self.submodules = None
-        self.inputs = None
-        self.output = None
+        self.in_buffer = None
+
+        # None for out buffer implies there are no out-ports
+        self.out_buffer = None
         self.specials = None
         self.comb = None
         self.sync = None
@@ -162,10 +169,14 @@ class MigenNodeTemplate(abc.ABC):
         self.__class__.migen_body(self=self._dut, template=self)
         self.rename_port_signals()
 
-        # Make all outputs named, as it's needed for verilog
-        self.ForkedOutsT, self.ForkedOuts = make_forked_return(
-            {name: type_ for name, (_, type_) in self._dut.out_ports.items()}
-        )
+        if len(self._dut.out_ports.items()) > 0:
+            # Make all outputs named, as it's needed for verilog
+            self._outputs, self._ForkedOutput = make_forked_return(
+                {name: type_ for name, (_, type_)
+                 in self._dut.out_ports.items()}
+            )
+        else:
+            self._outputs = Void
 
         # define a stimulus generator
         self._tb = self.tb_generator(tb_num_iter)
@@ -174,13 +185,20 @@ class MigenNodeTemplate(abc.ABC):
         self._sim_object = None
         atexit.register(self.cleanup)
 
-    def add_pa_in_port(self, name: str, t: DOptional):
+        # Merge with or create a new NodeTemplate
+        inputs = OrderedDict()
+        for name, (_, type_) in self._dut.in_ports.items():
+            inputs[name] = type_
+        NodeTemplate.merge_migenblock(node_template, self, inputs,
+                                      self._outputs)
+
+    def add_pa_in_port(self, name: str, t: Optional):
         """Add input protocol adaptor v2.
 
         Parameters
         ----------
-        t : DOptional
-            Data type must be marked with DOptional.
+        t : Optional
+            Data type must be marked with Optional.
 
         Returns
         -------
@@ -188,13 +206,13 @@ class MigenNodeTemplate(abc.ABC):
 
 
         .. todo::
-            Possibly use DSize for data/valid/ready instead of int.
+            Possibly use Size for data/valid/ready instead of int.
         """
-        if isinstance(t, DOptional):
+        if isinstance(t, Optional):
             df_t = t.type
         else:
             raise DeltaTypeError('Migen works only with optional inputs.\n'
-                                 'Please use DOptional wrapper.')
+                                 'Please use Optional wrapper.')
 
         in_port = migen.Record([
             ("data", df_t.size.val, migen.DIR_M_TO_S),
@@ -220,11 +238,11 @@ class MigenNodeTemplate(abc.ABC):
 
 
         .. todo::
-            Possibly use DSize for data/valid/ready instead of int.
+            Possibly use Size for data/valid/ready instead of int.
         """
         df_t = as_delta_type(t)
-        if isinstance(df_t, DOptional):
-            raise DeltaTypeError('out_port cannot be DOptional')
+        if isinstance(df_t, Optional):
+            raise DeltaTypeError('out_port cannot be Optional')
 
         out_port = migen.Record([
             ("data", df_t.size.val, migen.DIR_S_TO_M),
@@ -286,13 +304,18 @@ class MigenNodeTemplate(abc.ABC):
         else:
             return self._sim_object
 
-    @abc.abstractmethod
+    @abstractmethod
     def migen_body(self, template):
         """This is where user defines the combinational and synchronous logic
         that describes the node.
         """
         pass
 
+    def construct_body(self):
+        """Construct the ``PyMigenBody`` this is a template for.
+        """
+        return PyMigenBody(self._py_sim_body, self, self.latency, self._tags)
+
     def tb_generator(self, tb_num_iter: int = None):
         """Generator, a.k.a. testbench.
 
@@ -321,7 +344,7 @@ class MigenNodeTemplate(abc.ABC):
                     break
 
             self.log.debug(f"all _dut.in_ports ready: {is_module_ready}")
-            self.log.debug(f"inputs available: {bool(self.inputs)}")
+            self.log.debug(f"inputs available: {bool(self.in_buffer)}")
 
             # 2. If yes: transfer data inputs -> in_ports
             #
@@ -329,10 +352,10 @@ class MigenNodeTemplate(abc.ABC):
             # Currently all inputs are considered valid as long as they are
             # provided. This is not the case in general, for instance
             # the input can have all sort of junk if valid == False.
-            if bool(self.inputs) and is_module_ready:
+            if bool(self.in_buffer) and is_module_ready:
                 for name, (port, _) in self._dut.in_ports.items():
-                    if name in self.inputs:
-                        data = self.inputs[name]
+                    if name in self.in_buffer:
+                        data = self.in_buffer[name]
                         if data is None:
                             data = 0
                             valid = 0
@@ -360,17 +383,20 @@ class MigenNodeTemplate(abc.ABC):
             # 4. Transfer data out_ports -> output
             #
             # If output data for a channel is not valid,
-            #  the corresponding output is set to None
-            out_tmp = {}
-            for name, (port, _) in self._dut.out_ports.items():
-                if (yield port.valid):
-                    out_tmp[name] = yield port.data
-                else:
-                    out_tmp[name] = None
-            self.output = self.ForkedOuts(**out_tmp)
+            # the corresponding output is set to None
+            if len(self._dut.out_ports.items()) > 0:
+                out_tmp = {}
+                for name, (port, _) in self._dut.out_ports.items():
+                    if (yield port.valid):
+                        out_tmp[name] = yield port.data
+                    else:
+                        out_tmp[name] = None
+                self.out_buffer = self._ForkedOutput(**out_tmp)
+
+                self.log.debug(f"retrieved data output={self.out_buffer}")
+            else:
+                self.log.debug(f"no out_ports => no output")
 
-            self.log.debug(
-                f"retrieved data output={self.output}")
             self.log.debug(f"end of iteration {tb_iter}")
 
             # 5. Evaluate one clock cycle
@@ -378,7 +404,7 @@ class MigenNodeTemplate(abc.ABC):
 
             # 6. Status after the cycle
             self.log.debug(f'<<<<<<<<<<< STATUS AFTER YIELD IN ITER {tb_iter}')
-            self.log.debug(f'{self.inputs=}')
+            self.log.debug(f'{self.in_buffer=}')
             self.log.debug('in_ports:')
             for name, (port, _) in self._dut.in_ports.items():
                 data = yield port.data
@@ -397,7 +423,7 @@ class MigenNodeTemplate(abc.ABC):
                 valid = yield port.valid
                 ready = yield port.ready
                 self.log.debug(f'{name=}: {data=} {valid=} {ready=}')
-            self.log.debug(f'{self.output=}')
+            self.log.debug(f'{self.out_buffer=}')
             self.log.debug('>>>>>>>>>>>')
 
             tb_iter += 1
@@ -439,14 +465,14 @@ class MigenNodeTemplate(abc.ABC):
         """This method is handled over to the node body constructor, and it
         will be evaluated by the testbench at each clock cycle.
         """
-        self.inputs = kwargs  # enqueue the input
-        self.output = None
+        self.in_buffer = kwargs  # enqueue the input
+        self.out_buffer = None
 
         # runs until the generator terminates.
         # for this impl, this will happen when inputs is empty
         self._clock()
 
-        return self.output
+        return self.out_buffer
 
     def call(self, *args, **kwargs):
         """This performs the same action as ``Interactive.call``,
@@ -478,24 +504,11 @@ class MigenNodeTemplate(abc.ABC):
         .. todo::
             `args` are being ignored => raise an error if passed.
         """
-        if DeltaGraph.stack():
-            # there is currently an active DeltaGraph so use node factory
-            current_graph = DeltaGraph.current_graph()
 
-            in_params = dict()
-            for name, (_, type_) in self._dut.in_ports.items():
-                in_params[name] = type_
-            retval = py_method_node_factory(*args,
-                                            graph=current_graph,
-                                            node_func=self._py_sim_body,
-                                            obj=self,
-                                            in_params=in_params,
-                                            out_type=self.ForkedOutsT,
-                                            is_migen=True,
-                                            name=self.name,
-                                            latency=None,
-                                            lvl=self.log.level,
-                                            **kwargs)
+        from .._delta_graph import DeltaGraph
+        if DeltaGraph.stack():
+            graph = DeltaGraph.current_graph()
+            retval = self._call(graph, *args, **kwargs)
             self.module_name = retval.name
         else:
             # it does only one clock, so you need to place it in a loop
